package org.greenstone.gsdl3.gs3build.metadata;

import java.io.PrintWriter;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Iterator;

import java.sql.SQLException;
import java.sql.ResultSet;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.greenstone.gsdl3.gs3build.doctypes.DocumentInterface;

import org.greenstone.gsdl3.gs3build.database.*;

public class METSDescriptiveSet
{ 
    Map children;
    Map childrenById;
    DescriptiveIdentifierFactory identifierFactory;
    
    class DescriptiveIdentifierFactory extends AbstractIdentifierFactory
    { 
	public DescriptiveIdentifierFactory()
	{ 
	    super("DM");
	}
    }
    
    public METSDescriptiveSet()
    { 
	this.children = new HashMap();
	this.childrenById = new HashMap();
	
	this.identifierFactory = new DescriptiveIdentifierFactory();
	
	this.createDescriptive("default");
    }
    
    /**
     *  Create a new descriptive child, adding it automatically to this descriptive set.
     *
     *  @param <code>String</code> a name to describe the descriptive set.
     */
    public METSDescriptive createDescriptive(String name)
    { 
	METSDescriptive descriptive = new METSDescriptive(this.identifierFactory.getNextIdentifier(), name);
	
	this.children.put(name, descriptive);
	this.childrenById.put(descriptive.getID(), descriptive);
	return descriptive;
    }
    
    /**
     *  Add a descriptive item to this set.
     *
     *  @param <code>METSDescriptive</code> the descriptive set
     */
    public void addDescriptive(METSDescriptive metadata)
    { 
	// insert the item into both maps...
	this.children.put(metadata.getName(), metadata);
	this.childrenById.put(metadata.getID(), metadata);
	
	// when a descriptive block is added, check that it hasn't been used before...
	//this.identifierFactory.noteIdentifier(metadata.getID());
	this.identifierFactory.noteIdentifier(metadata.getName());
    }
    
    /**
     *  Get a descriptive item by its name(GROUPID in METS file)
     */
    public METSDescriptive getDescriptive(String name)
    { 
	
	return (METSDescriptive) this.children.get(name);
    }
    
    /**
     *  Get a descriptive item by its identifier
     */
    public METSDescriptive getDescriptiveById(String id)
    { 
	return (METSDescriptive) this.childrenById.get(id);
    }
    
    public void addNamespace(String descriptiveName, METSNamespace namespace)
    { 
	METSDescriptive descriptive = this.getDescriptive(descriptiveName);
	if (descriptive != null){
	    descriptive.addNamespace(namespace);
	}
    }
    
    public METSNamespace getNamespace(String descriptiveName, String namespace)
    { 
	METSDescriptive descriptive = this.getDescriptive(descriptiveName);
	if (descriptive != null){
	    return descriptive.getNamespace(namespace);
	}
	return null;
    }
    
    public METSNamespace getOpenNamespace(String descriptiveName, String namespace)
    { 
	METSDescriptive descriptive = this.getDescriptive(descriptiveName);
	
	if (descriptive != null){
	    return descriptive.getOpenNamespace(namespace);
	}
	return  null;
    }
    
    public void addMetadata(String descriptiveName, String namespace, String label, String value)
    { 
	METSDescriptive descriptive = this.getDescriptive(descriptiveName);
	
	if (descriptive != null){
	    descriptive.addMetadata(namespace, label, value);
	}
    }
    
    public void setMetadata(String descriptiveName, String namespace, String label, String value)
    { 
	METSDescriptive descriptive = this.getDescriptive(descriptiveName);
	
	if (descriptive != null){
	    descriptive.setMetadata(namespace, label, value);
	}
    }
    
    public void removeMetadata(String descriptiveName, String namespace, String label)
    { 
	METSDescriptive descriptive = this.getDescriptive(descriptiveName);
	
	if (descriptive != null){
	    descriptive.removeMetadata(namespace, label);
	}
    }
    
    public void removeAllMetadata(String namespace, String label)
    { 
	Iterator descriptiveIter = this.children.values().iterator();
	
	while (descriptiveIter.hasNext()) {
	    METSDescriptive descriptive = (METSDescriptive) descriptiveIter.next();
	    
	    descriptive.removeMetadata(namespace, label);
	}
    }
    
    public List getMetadata(String descriptiveName, String namespace, String label)
    { 
	METSDescriptive descriptive = this.getDescriptive(descriptiveName);
	
	if (descriptive != null){
	    return descriptive.getMetadata(namespace, label);
	}
	return null;
    }
    
    public void write(PrintWriter writer)
    { 
	Iterator groups = this.children.values().iterator();
	
	while (groups.hasNext()){
	    METSDescriptive group = (METSDescriptive) groups.next();
	    
	    group.write(writer);
	}
    }
    
    
    public boolean writeSQL(DocumentInterface document, GS3SQLConnection connection)
    { 
	Iterator groups = this.children.values().iterator();
	
	while (groups.hasNext()){
	    METSDescriptive group = (METSDescriptive) groups.next();
	    
	    if (!group.writeSQL(document, connection)){
		return false;
	    }
	}
	return true;
    }

    public static METSDescriptiveSet parseXML(NodeList dmdSecs) 
    {
	METSDescriptiveSet set = new METSDescriptiveSet();
	
	for (int g = 0; g < dmdSecs.getLength(); g ++) {
	    METSDescriptive metadata = METSDescriptive.parseXML((Element) dmdSecs.item(g));
	    
	    set.addDescriptive(metadata);
	}
    
	System.err.println("******** Away to do transfer!!!!!!");
	  
	METSDescriptive top_level_metadata = set.getDescriptive("1");
	METSDescriptive tl2_metadata = set.getDescriptive("1");
	METSDescriptive opo_metadata = set.getDescriptive("1.1");
	METSDescriptive default_metadata = set.getDescriptive("default");
	
	if (top_level_metadata != null) {
	    if (default_metadata != null) {
		top_level_metadata.copy_metadata(default_metadata);
	    }
	    else {
		System.err.println("Warning: Could not find default metadata");
	    }
	}
	else {
	    System.err.println("Warning: Could not find top level metadata for groupID=1");
	}
	return set;
    } 
    
    public static METSDescriptiveSet readSQL(DocumentInterface document, GS3SQLConnection connection)
    {
	METSDescriptiveSet set = new METSDescriptiveSet();
	
	GS3SQLSelect select = new GS3SQLSelect("metadata");
	select.addField("*");
	GS3SQLWhereItem whereItem = new GS3SQLWhereItem("DocID", "=", document.getID().toString());
	select.setWhere(new GS3SQLWhere(whereItem));
	connection.execute(select.toString());
	
	// start going through the matching metadata blocks
	try {
	    ResultSet resultSet = connection.getResultSet();
	    resultSet.first();
	    do {
		METSDescriptive descriptive = METSDescriptive.readSQL(document, connection, resultSet);
		if (descriptive != null) {
		    set.addDescriptive(descriptive);
		}
		else {
		    System.out.println("Null descriptive");
		}
	    } while (resultSet.next());
	}
	catch (SQLException sqlEx) {
	    System.out.println(sqlEx);
	    System.exit(1);
	}
	
	return set;
    }
}

